@@ -19,6 +19,12 @@
 #'   If only a single value is passed into the function,
 #'   the RD will similarly be estimated at that bandwidth, half that bandwidth, 
 #'   and twice that bandwidth.
+#' @param front.bw A numeric vector specifying the bandwidths at which to estimate the RD for each
+#'   of three effects models in the frontier method. If NA, front.bw will be determined by cross validation.
+#' @param m The number of uniformly-at-random samples to draw as search candidates for front.bw 
+#'   if not given.
+#' @param k An integer specifying the number of folds for cross validation to determine front.bw
+#'   if not given. 
 #' @param kernel A string specifying the kernel to be used in the local linear fitting. 
 #'   \code{"triangular"} kernel is the default and is the "correct" theoretical kernel to be 
 #'   used for edge estimation as in RDD (Lee and Lemieux, 2010). Other options are 
@@ -41,7 +47,8 @@
 #' @param local The range of neighboring points around the cutoff on the 
 #'   standardized scale on each assignment variable, which is a positive number.
 #' @param ngrid The number of non-zero grid points on each assignment variable,
-#'   which is also the number of zero grid points on each assignment variable.
+#'   which is also the number of zero grid points on each assignment variable. Value used in 
+#'   Wong, Steiner and Cook (2013) is 2500, which may cause long computational time.
 #' @param margin The range of grid points beyond the minimum and maximum
 #'   of sample points on each assignment variable.
 #' @param boot The number of bootstrap samples to obtain standard deviation of estimates.
@@ -54,8 +61,12 @@
 #'   if X1 is less than its cutoff, \code{"leq"} means treatment is assigned 
 #'   if X1 is less than or equal to its cutoff.
 #'   The 2nd entry is for X2.
+#' @param stop.on.error Logical. If \code{TRUE} (the default), removes bootstraps which cause
+#'   error in the \code{integrate} function, and resample till the specified number of 
+#'   bootstrap samples are acquired.
 #'
-#' @return \code{rd_impute} returns an object of \link{class} "\code{mrd}".
+#' @return \code{mrd_impute} returns an object of \link{class} "\code{mrd}", or "\code{mrdi}" for
+#' \code{"front"} method.
 #'
 #' @references Stata: 64 mi estimate - Estimation using multiple imputations
 #'
@@ -67,22 +78,24 @@
 #' @export
 #'
 #' @examples
-#' x1 <- runif(1000, -1, 1)
-#' x2 <- runif(1000, -1, 1)
-#' cov <- rnorm(1000)
-#' y <- 3 + 2 * (x1 >= 0) + 3 * cov + 10 * (x2 >= 0) + rnorm(1000)
-#' group <- rep(1:10, each = 100)
+#' x1 <- runif(300, -1, 1)
+#' x2 <- runif(300, -1, 1)
+#' cov <- rnorm(300)
+#' y <- 3 + 2 * (x1 >= 0) + 3 * cov + 10 * (x2 >= 0) + rnorm(300)
+#' imp <- rep(1:3, each = 100)
+#' # all examples below have smaller numbers of m to keep run-time low
 #' # centering
-#' mrd_impute(y ~ x1 + x2 | cov, impute = group, method = "center", t.design = c("geq", "geq"))
+#' mrd_impute(y ~ x1 + x2 | cov, impute = imp, method = "center", t.design = c("geq", "geq"), m = 3)
 #' # univariate
-#' mrd_impute(y ~ x1 + x2 | cov, impute = group, method = "univ", t.design = c("geq", "geq"))
+#' mrd_impute(y ~ x1 + x2 | cov, impute = imp, method = "univ", t.design = c("geq", "geq"), m = 3)
 #' # frontier
-#' mrd_impute(y ~ x1 + x2 | cov, impute = group, method = "front", t.design = c("geq", "geq"))
+#' mrd_impute(y ~ x1 + x2 | cov, impute = imp, method = "front", t.design = c("geq", "geq"), m = 3)
 
 mrd_impute <- function(formula, data, subset = NULL, cutpoint = NULL, bw = NULL, 
+  front.bw = NA, m = 10, k = 5,
   kernel = "triangular", se.type = "HC1", cluster = NULL, impute = NULL, verbose = FALSE, 
-  less = FALSE, est.cov = FALSE, est.itt = FALSE, local = 0.15, ngrid = 2500, margin = 0.03, 
-  boot = NULL, method = c("center", "univ", "front"), t.design = NULL) {
+  less = FALSE, est.cov = FALSE, est.itt = FALSE, local = 0.15, ngrid = 250, margin = 0.03, 
+  boot = NULL, method = c("center", "univ", "front"), t.design = NULL, stop.on.error = TRUE) {
  
   if (is.null(t.design)){
     stop("Specify t.design.")
@@ -117,27 +130,35 @@ mrd_impute <- function(formula, data, subset = NULL, cutpoint = NULL, bw = NULL,
     if (is.null(subset)) {
       if (missing(data)) {
         curr_mod <- mrd_est(formula = formula, subset = imp_sub, cutpoint = cutpoint, bw = bw,
+          front.bw = front.bw, m = m, k = k,
           kernel = kernel, se.type = se.type, cluster = cluster, verbose = verbose, less = less, 
           est.cov = est.cov, est.itt = est.itt, local = local, ngrid = ngrid, margin = margin, 
-          boot = boot, method = method, t.design = t.design)
+          boot = boot, method = method, t.design = t.design, 
+          stop.on.error = stop.on.error)
       } else {
         curr_mod <- mrd_est(formula = formula, data = data, subset = imp_sub, cutpoint = cutpoint, 
-          bw = bw, kernel = kernel, se.type = se.type, cluster = cluster, verbose = verbose, 
+          bw = bw, front.bw = front.bw, m = m, k = k,
+          kernel = kernel, se.type = se.type, cluster = cluster, verbose = verbose, 
           less = less, est.cov = est.cov, est.itt = est.itt, local = local, ngrid = ngrid, 
-          margin = margin, boot = boot, method = method, t.design = t.design)
+          margin = margin, boot = boot, method = method, t.design = t.design, 
+          stop.on.error = stop.on.error)
       }
       
     } else {
       if (missing(data)) {
         curr_mod <- mrd_est(formula = formula, subset = (subset & imp_sub), cutpoint = cutpoint, 
-          bw = bw, kernel = kernel, se.type = se.type, cluster = cluster, verbose = verbose, 
+          bw = bw, front.bw = front.bw, m = m, k = k,
+          kernel = kernel, se.type = se.type, cluster = cluster, verbose = verbose, 
           less = less, est.cov = est.cov, est.itt = est.itt, local = local, ngrid = ngrid, 
-          margin = margin, boot = boot, method = method, t.design = t.design)
+          margin = margin, boot = boot, method = method, t.design = t.design,
+          stop.on.error = stop.on.error)
       } else {
         curr_mod <- mrd_est(formula = formula, data = data, subset = (subset & imp_sub), 
-          cutpoint = cutpoint, bw = bw, kernel = kernel, se.type = se.type, cluster = cluster,
+          cutpoint = cutpoint, bw = bw, front.bw = front.bw, m = m, k = k,
+          kernel = kernel, se.type = se.type, cluster = cluster,
           verbose = verbose, less = less, est.cov = est.cov, est.itt = est.itt, local = local, 
-          ngrid = ngrid, margin = margin, boot = boot, method = method, t.design = t.design)
+          ngrid = ngrid, margin = margin, boot = boot, method = method, t.design = t.design,
+          stop.on.error = stop.on.error)
       }
       
     }
@@ -235,17 +256,25 @@ mrd_impute <- function(formula, data, subset = NULL, cutpoint = NULL, bw = NULL,
       if (i == 1) {
         o$front$tau_MRD$call <- front_MRD$call
         
-        est_res$front_MRD <- matrix(NA, nrow = num_imp, ncol = 9)
-        colnames(est_res$front_MRD) <- names(front_MRD$est)
-        d_res$front_MRD <- matrix(NA, nrow = num_imp, ncol = 9)
-        colnames(d_res$front_MRD) <- names(front_MRD$d)
-        se_res$front_MRD <- matrix(NA, nrow = num_imp, ncol = 9)
-        colnames(se_res$front_MRD) <- names(front_MRD$se)
+        M <- matrix(NA, nrow = num_imp, ncol = 9)
+        colnames(M) <- colnames(front_MRD$est)
+        est_res$front_MRD <- list('Param' = M, 'bw' = M, 'Half-bw' = M, 'Double-bw' = M)
+        d_res$front_MRD <- list('Param' = M, 'bw' = M, 'Half-bw' = M, 'Double-bw' = M)
+        se_res$front_MRD <- list('Param' = M, 'bw' = M, 'Half-bw' = M, 'Double-bw' = M)
       }
 
-      est_res$front_MRD[i, ] <- front_MRD$est
-      d_res$front_MRD[i, ] <- front_MRD$d
-      se_res$front_MRD[i, ] <- front_MRD$se
+      est_res$front_MRD$'Param'[i, ] <- front_MRD$est['Param',]
+      est_res$front_MRD$'bw'[i, ] <- front_MRD$est['bw',]
+      est_res$front_MRD$'Half-bw'[i, ] <- front_MRD$est['Half-bw',]
+      est_res$front_MRD$'Double-bw'[i, ] <- front_MRD$est['Double-bw',]
+      d_res$front_MRD$'Param'[i, ] <- front_MRD$d['Param',]
+      d_res$front_MRD$'bw'[i, ] <- front_MRD$d['bw',]
+      d_res$front_MRD$'Half-bw'[i, ] <- front_MRD$d['Half-bw',]
+      d_res$front_MRD$'Double-bw'[i, ] <- front_MRD$d['Double-bw',]      
+      se_res$front_MRD$'Param'[i, ] <- front_MRD$se['Param',]
+      se_res$front_MRD$'bw'[i, ] <- front_MRD$se['bw',]
+      se_res$front_MRD$'Half-bw'[i, ] <- front_MRD$se['Half-bw',]
+      se_res$front_MRD$'Double-bw'[i, ] <- front_MRD$se['Double-bw',]
     }
 
   }
@@ -341,31 +370,41 @@ mrd_impute <- function(formula, data, subset = NULL, cutpoint = NULL, bw = NULL,
   }
   
   if ("front" %in% method) {
-    Q <- colMeans(est_res$front_MRD)
-    D <- colMeans(d_res$front_MRD)
-    U <- colMeans(se_res$front_MRD)
-    B <- apply(se_res$front_MRD, 2, var)
+    Q <- rbind(colMeans(est_res$front_MRD$'Param'), colMeans(est_res$front_MRD$'bw'),
+               colMeans(est_res$front_MRD$'Half-bw'), colMeans(est_res$front_MRD$'Double-bw'))
+    D <- rbind(colMeans(d_res$front_MRD$'Param'), colMeans(d_res$front_MRD$'bw'),
+               colMeans(d_res$front_MRD$'Half-bw'), colMeans(d_res$front_MRD$'Double-bw'))
+    U <- rbind(colMeans(se_res$front_MRD$'Param'), colMeans(se_res$front_MRD$'bw'),
+               colMeans(se_res$front_MRD$'Half-bw'), colMeans(se_res$front_MRD$'Double-bw'))
+    B <- rbind(apply(se_res$front_MRD$'Param', 2, var), apply(se_res$front_MRD$'bw', 2, var),
+               apply(se_res$front_MRD$'Half-bw', 2, var), apply(se_res$front_MRD$'Double-bw', 2, var))
     V <- U + (1 + 1 / num_imp) * B
+    list.names = c('Param', 'bw', 'Half-bw', 'Double-bw')
+    rownames(Q) = rownames(D) = rownames(U) = rownames(B) = rownames(V) <- list.names
     
     o$front$tau_MRD$est <- Q
     o$front$tau_MRD$d <- D
     o$front$tau_MRD$se <- sqrt(V)
-    o$front$tau_MRD$z <- unname(o$front$tau_MRD$est/o$front$tau_MRD$se)
+    o$front$tau_MRD$z <- o$front$tau_MRD$est/o$front$tau_MRD$se
     
     r <- (1 + 1 / num_imp) * B / U
     df <- (num_imp - 1) * (1 + 1 / r)^2
     o$front$tau_MRD$df <- df
     
     o$front$tau_MRD$p <- 2 * pt(abs(o$front$tau_MRD$z), df = df, lower.tail = FALSE)
-    o$front$tau_MRD$ci <- unname(c(
-      o$front$tau_MRD$est - qt(0.975, df = df) * o$front$tau_MRD$se, 
-      o$front$tau_MRD$est + qt(0.975, df = df) * o$front$tau_MRD$se))
-    o$front$tau_MRD$ci <- matrix(o$front$tau_MRD$ci, ncol = 2)
+    ci <- matrix(NA, 8, 9)
+    ci[c(1, 3, 5, 7),] <- o$front$tau_MRD$est - qt(0.975, df = df) * o$front$tau_MRD$se
+    ci[c(2, 4, 6, 8),] <- o$front$tau_MRD$est + qt(0.975, df = df) * o$front$tau_MRD$se
+    rownames(ci) <- rep(c('2.5%', '97.5%'), 4)
+    colnames(ci) <- colnames(o$front$tau_MRD$est)
+    o$front$tau_MRD$ci <- ci
     
-    o$front$tau_MRD$w <- rep(NA, 2)   
+    o$front$tau_MRD$w <- front_MRD$w   
+    o$front$tau_MRD$front.bw <- front_MRD$front.bw
 
     o$front$tau_MRD$impute <- TRUE
-    class(o$front$tau_MRD) <- "mfrd"                
+    class(o) = "mrdi"
+    class(o$front$tau_MRD) <- "mfrdi"                
   }
 
   return(o)
