@@ -2,6 +2,8 @@
 #' 
 #' \code{rd_type} cross-tabulates observations based on (1) a binary treatment and 
 #' (2) one or two assignments and their cutoff values.
+#' This is an internal function and is typically not directly invoked by the user. 
+#' It can be accessed using the triple colon, as in rddapp:::rd_type().
 #'
 #' @param data A \code{data.frame} with each row representing an observation.
 #' @param treat The name of a numeric variable (treated = positive values).
@@ -19,6 +21,13 @@
 #' @importFrom stats reshape xtabs
 #'
 #' @include treat_assign.R
+#' 
+#' @examples
+#' x <- runif(1000, -1, 1)
+#' cov <- rnorm(1000)
+#' y <- 3 + 2 * x + 3 * cov + 10 * (x >= 0) + rnorm(1000)
+#' df <- data.frame(cbind(y, x, t = x>=0))
+#' rddapp:::rd_type(df, 't', 'x', 0, 'geq')
 
 rd_type <- function(data, treat, assign_1, cutoff_1, operator_1 = NULL, 
   assign_2 = NULL, cutoff_2 = NULL, operator_2 = NULL) {
@@ -55,21 +64,22 @@ rd_type <- function(data, treat, assign_1, cutoff_1, operator_1 = NULL,
     
     xdf <- reshape(xdf, direction = "wide", v.names = "freq", timevar = "t", idvar = "a1")
     
-    # GET COEFFICIENTS OF X>=C FROM ESTIMATE FIRST STAGE OF 2SLS
-    
-    fml <- substitute(
-      trt ~ 0 
-      + I(1-tstar_1) 
-      + tstar_1 
-      + I((X - C) * (1 - tstar_1)) 
-      + I((X - C) * (tstar_1)), 
-      env = list(
-        trt = as.name(treat), 
-        tstar_1 = as.name("tstar_1"),
-        C = cutoff_1, 
-        X = as.name(assign_1)))
-    
-    xdf <- cbind(xdf, round(coef(lm(fml, data))[1:2], 3))
+    xdf$prob =  xdf$freq.TRUE/(xdf$freq.FALSE + xdf$freq.TRUE)
+    # # GET COEFFICIENTS OF X>=C FROM ESTIMATE FIRST STAGE OF 2SLS
+    # 
+    # fml <- substitute(
+    #   trt ~ 0 
+    #   + I(1-tstar_1) 
+    #   + tstar_1 
+    #   + I((X - C) * (1 - tstar_1)) 
+    #   + I((X - C) * (tstar_1)), 
+    #   env = list(
+    #     trt = as.name(treat), 
+    #     tstar_1 = as.name("tstar_1"),
+    #     C = cutoff_1, 
+    #     X = as.name(assign_1)))
+    # 
+    # xdf <- cbind(xdf, round(coef(lm(fml, data))[1:2], 3))
     
     colnames(xdf) <- c("A1", "Control", "Treat", "Prob.")
     
@@ -118,27 +128,28 @@ rd_type <- function(data, treat, assign_1, cutoff_1, operator_1 = NULL,
     xdf <- reshape(xdf, direction = "wide", v.names = "freq", timevar = "t", idvar = c("a1", "a2"))
     is.na(xdf)[c(2, 4), 1] <- TRUE
     
-    # GET COEFFICIENTS OF X>=C FROM ESTIMATE FIRST STAGE OF 2SLS
-    
-    fml <- substitute(
-      trt ~ 0 
-      + I((1-tstar_1) * (1-tstar_2)) 
-      + I((1-tstar_1) * tstar_2)  
-      + I(tstar_1 * (1-tstar_2))  
-      + I(tstar_1 * tstar_2)    
-      + I((X1 - C1) * (X2 - C2) * (1-tstar_1) * (1-tstar_2)) 
-      + I((X1 - C1) * (X2 - C2) * (1-tstar_1) * tstar_2)   
-      + I((X1 - C1) * (X2 - C2) * tstar_1 * (1-tstar_2))   
-      + I((X1 - C1) * (X2 - C2) * tstar_1 * tstar_2),
-      env = list(
-        trt = as.name(treat), 
-        C1 = cutoff_1, X1 = as.name(assign_1), tstar_1 = as.name("tstar_1"), 
-        C2 = cutoff_2, X2 = as.name(assign_2), tstar_2 = as.name("tstar_2")
-      )
-    )
-
+    xdf$prob =  xdf$freq.TRUE/(xdf$freq.FALSE + xdf$freq.TRUE)
+    # # GET COEFFICIENTS OF X>=C FROM ESTIMATE FIRST STAGE OF 2SLS
+    # 
+    # fml <- substitute(
+    #   trt ~ 0 
+    #   + I((1-tstar_1) * (1-tstar_2)) 
+    #   + I((1-tstar_1) * tstar_2)  
+    #   + I(tstar_1 * (1-tstar_2))  
+    #   + I(tstar_1 * tstar_2)    
+    #   + I((X1 - C1) * (X2 - C2) * (1-tstar_1) * (1-tstar_2)) 
+    #   + I((X1 - C1) * (X2 - C2) * (1-tstar_1) * tstar_2)   
+    #   + I((X1 - C1) * (X2 - C2) * tstar_1 * (1-tstar_2))   
+    #   + I((X1 - C1) * (X2 - C2) * tstar_1 * tstar_2),
+    #   env = list(
+    #     trt = as.name(treat), 
+    #     C1 = cutoff_1, X1 = as.name(assign_1), tstar_1 = as.name("tstar_1"), 
+    #     C2 = cutoff_2, X2 = as.name(assign_2), tstar_2 = as.name("tstar_2")
+    #   )
+    # )
+    #  
     # coef(lm(fml, data))
-    xdf <- cbind(xdf, round(coef(lm(fml, data))[1:4], 3))
+    # xdf <- cbind(xdf, round(coef(lm(fml, data))[1:4], 3))
     
     colnames(xdf) <- c("A1", "A2", "Control", "Treat", "Prob.")
   }
@@ -226,7 +237,7 @@ rd_type <- function(data, treat, assign_1, cutoff_1, operator_1 = NULL,
     
     if ((all(!dmat[1:3, 1]) & dmat[4, 1] & !dmat[4, 2]) 
       | (all(!dmat[2:4, 1]) & dmat[1, 1] & !dmat[1, 2])) {
-      if (all((xtabs(tr ~ tstar_1 + tstar_2, data) > 0) == matrix(c(0, 1, 1, 1), 
+      if (all((xtabs(paste0(treat, " ~ tstar_1 + tstar_2"), data) > 0) == matrix(c(0, 1, 1, 1), 
         nrow = 2, byrow = TRUE))) {
         return(list(crosstab = xdf, type = "SHARP FRONTIER"))
       } 
