@@ -1,5 +1,6 @@
 sensitivity_analysisUI = function(id){
   ns = NS(id)
+  ## compute good default yrange for bw sensitivity plot
   tagList(
     p(),
     fluidRow(
@@ -13,7 +14,7 @@ sensitivity_analysisUI = function(id){
                 span(class='input-group-addon','CI'),
                 eval({
                   tag=numericInput(ns('cutsens_level'), label = NULL, min = 0, max = 99.9, 
-                    value = 95, width = '65px')
+                    value = 95, width = '50px')
                   
                   tag$attribs$class = paste(tag$attribs$class, 'input-group-sm')
                   tag
@@ -106,7 +107,7 @@ sensitivity_analysisUI = function(id){
                 span(class='input-group-addon','CI'),
                 eval({
                   tag=numericInput(ns('bwsens_level'), label = NULL, min = 0, max = 99.9, 
-                    value = 95, width = '65px')
+                    value = 95, width = '50px')
                   tag$attribs$class = paste(tag$attribs$class, 'input-group-sm')
                   tag
                 }),
@@ -148,7 +149,10 @@ sensitivity_analysisUI = function(id){
                 
               )
             ),
-            uiOutput(ns('bwsens_controls'))
+            h6("X-axis"),
+            uiOutput(ns('bwsens_controls')),
+            h6("Y-axis"),
+            uiOutput(ns('bwsens_ylims'))
           )
         )
       )
@@ -157,7 +161,6 @@ sensitivity_analysisUI = function(id){
 }
 
 sensitivity_analysis =  function(input, output, session, dataframe, parameter, result){
-  
   ## CUTOFFS
   
   # UPDATE CUTOFF RANGE INPUT
@@ -198,7 +201,6 @@ sensitivity_analysis =  function(input, output, session, dataframe, parameter, r
   })
   
   cut_sens_summary = eventReactive(input$cutsens_simulate, ignoreNULL = T, {
-    
     req(class(result$model()) %in% c('rd','mrd'))
     req(input$cutsens_range_min, input$cutsens_range_max, input$cutsens_step)
     
@@ -262,7 +264,7 @@ sensitivity_analysis =  function(input, output, session, dataframe, parameter, r
       length(input$cutsens_models) > 0,
       all(input$cutsens_models %in% cut_sens_summary()$model)
     )
-    
+
     sens_plot(cut_sens_summary(),
       level = input$cutsens_level / 100,
       x = isolate(input$cutsens_which_cutoff),
@@ -319,7 +321,7 @@ sensitivity_analysis =  function(input, output, session, dataframe, parameter, r
   output$cutsens_png = downloadHandler(
     filename = 'cutoff_sensitivity.png',
     content = function(file) {
-      png(file, width=700, height=600, res=150)
+      png(file, width=700, height=600, res=100)
       cutsens_plot()
       dev.off()
     }
@@ -360,6 +362,42 @@ sensitivity_analysis =  function(input, output, session, dataframe, parameter, r
     )
   })
   
+  ## Update y-limits for fig 4.2 per result
+  bwsens_est = reactiveVal()
+  bwsens_se = reactiveVal()
+  output$bwsens_ylims = renderUI({
+    ns = session$ns
+    req(class(result$model()) %in% c('rd','mrd'))
+    bwsens_est(if(class(result$model()) == 'rd'){
+      result$model()$est['Linear']
+    } else {
+      switch( input$bwsens_which_est,
+              'center' =  result$model()$center$tau_MRD$est['Linear'],
+              'univ1'=  result$model()$univ$tau_R$est['Linear'],
+              'univ2'=  result$model()$univ$tau_M$est['Linear']
+      )
+    })
+    bwsens_se(if(class(result$model()) == 'rd'){
+      result$model()$se['Linear']
+    } else {
+      switch( input$bwsens_which_est,
+              'center' =  result$model()$center$tau_MRD$se['Linear'],
+              'univ1'=  result$model()$univ$tau_R$se['Linear'],
+              'univ2'=  result$model()$univ$tau_M$se['Linear']
+      )
+    })
+    div(class = 'input-group',
+        span(class='input-group-addon', tags$small('min')),
+        numericInput(ns('bwsens_ymin'), label = NULL,
+                     value = round(bwsens_est()-3*bwsens_se(), 2),
+                     min =  -Inf, max = Inf, step = .01,  width = '95px'),
+        span(class='input-group-addon', tags$small('max')),
+        numericInput(ns('bwsens_ymax'), label = NULL,
+                     value =  round(bwsens_est()+3*bwsens_se(), 2),
+                     min = -Inf, max = Inf, step = .01,  width = '95px')
+    )
+  })
+
   
   bw_sens_summary = eventReactive(input$bwsens_simulate, ignoreNULL = T, {
     req(class(result$model()) %in% c('rd','mrd'))
@@ -400,16 +438,15 @@ sensitivity_analysis =  function(input, output, session, dataframe, parameter, r
       class(result$model()) %in% c('rd','mrd'),
       bw_sens_summary()
     )
-    yrange = bw_sens_summary()$est[grepl('origin',bw_sens_summary()$model)] + bw_sens_summary()$se[grepl('origin',bw_sens_summary()$model)] * qnorm(c(.005,.995))
-    
+
     sens_plot(bw_sens_summary(),
       level = input$bwsens_level / 100,
       x = 'bw',
-      yrange = yrange
+      yrange = c(input$bwsens_ymin, input$bwsens_ymax)
     )
     grid(col = 'black')
     abline(v = opt(), col = 'red3', lty = 2)
-    text(x = opt(), y = min(yrange), labels = 'optimal bandwidth', col = 'red3',  adj = c(-.1, 0))
+    text(x = opt(), y = min(input$bwsens_ymin), labels = 'optimal bandwidth', col = 'red3',  adj = c(-.1, 0))
     
     mtext('Bandwidth', side = 1, 2)
     mtext('Estimate', side = 2, 2)
@@ -455,7 +492,7 @@ sensitivity_analysis =  function(input, output, session, dataframe, parameter, r
   output$bwsens_png = downloadHandler(
     filename = 'bandwidth_sensitivity.png',
     content = function(file) {
-      png(file, width=700, height=600, res=150)
+      png(file, width=700, height=600, res=100)
       bwsens_plot()
       dev.off()
     }
